#
# This is the program you'd use if you wanted to generate the databases yourself.
# These are the same databases that the live site uses.
#

use Text::CSV::Simple;
use strict;
use Class::Struct;
use warnings;

use constant DEBUG => 0;

struct( point => [
	zip => '$',
	lat => '$',
	long => '$',
]);

# Load the input data file from CSV into memory. Pretty basic.
my $datafile = "USZIPCODESDATA.txt";
my $parser = Text::CSV::Simple->new;
$parser->want_fields(0, 1, 3, 4);
my @data = $parser->read_file($datafile);

my %states = ();

foreach (@data) {
	my $p = new point;
	$p->zip($_->[0]);
	$p->lat($_->[2]);
	$p->long($_->[3]);
	
	push(@{$states{$_->[1]}}, $p);
}

# This function has been designed to aid export to CSV, which you can then import into Excel.
if (DEBUG) {
	print "\"State Abbreviation\",\"Number of ZIP Codes\"\n";
	while (my ($stateAbbreviation, $pointArray) = each (%states)) {
		print "\"$stateAbbreviation\",\"" . scalar(@{ $pointArray} ) . "\"\n";
	}
}

# Time to build files. First create the file containing the averages and state abbreviations. This will help the program guess where you are.
mkdir("db");
open FILE, ">db/master.csv" or die $!;
print FILE "\"State Abbreviation\",\"Latitude\",\"Longitude\"\n";
while (my ($stateAbbreviation, $pointArray) = each (%states)) {
	my $avg_lat;
	my $avg_long;
	
	# Since we're looping through the master array anyways, let's go ahead and create
	# this individual state's database too.
	open SFILE, ">db/$stateAbbreviation.csv" or die $!;
	print SFILE "\"ZIP\",\"Latitude\",\"Longitude\"\n";
	foreach (@{ $pointArray }) {
		print SFILE "\"" . $_->zip . "\",\"" . $_->lat . "\",\"" . $_->long . "\"\n";
		$avg_lat += $_->lat;
		$avg_long += $_->long;
	}
	close SFILE;
	
	$avg_lat /= scalar(@{ $pointArray} );
	$avg_long /= scalar(@{ $pointArray} );
	
	print FILE "\"$stateAbbreviation\",\"$avg_lat\",\"$avg_long\"\n";
}
close FILE;

# And there you have it. I'm sure there are much more efficient algorithms out there, 
# but unfortunately I'm not a mathematician and I'm barely a programmer so this was
# the best practical thing I could muster up. Let me know if you have any
# constructive criticism!