# A big shoutout to http://jan.ucc.nau.edu/~cvm/latlon_formula.html for all the help with the math.
use Weather::Google;
use Text::CSV::Simple;
use strict;
use Geo::Coordinates::UTM;


###############################################################################
########################### BEGIN MAIN MATH ###################################
###############################################################################
# provides most accurate value of Pi supported by architecture.
use constant PI => 4 * atan2(1, 1);

# radius of earth.
#3963.1 statute miles
#3443.9 nautical miles
#6378 km
use constant R => 3963.1;

sub degreesToRadians {
	my $degrees = shift;
	return $degrees*(PI/180);
}

# use as distance(latitude1, longitude1, latitude2, longitude2)
sub distance {
	my $a1 = shift;
	my $b1 = shift;
	my $a2 = shift;
	my $b2 = shift;
	$a1 = degreesToRadians($a1);
	$b1 = degreesToRadians($b1);
	$a2 = degreesToRadians($a2);
	$b2 = degreesToRadians($b2);
	
	return &acos(cos($a1)*cos($b1)*cos($a2)*cos($b2) + cos($a1)*sin($b1)*cos($a2)*sin($b2) + sin($a1)*sin($a2)) * R;
}

sub acos {
   my($x) = @_;
   my $ret = atan2(sqrt(1 - $x**2), $x);
   return $ret;
}
###############################################################################
############################ END MAIN MATH ####################################
###############################################################################



my $datafile = "USZIPCODESDATA.txt";

my $location_latitude;
my $location_longitude;

if ($ARGV[0] eq "-mgrs") {
	my $location_mgrs = $ARGV[1] or die "No MGRS grid reference specified at command line.\n";
	($location_latitude, $location_longitude) = mgrs_to_latlon(23, $location_mgrs) or die "$!\n";
} elsif ($ARGV[0] eq "-utm") {
	my $zone = $ARGV[1] or die "No UTM zone (i.e. 17S) specified at command line.\n";
	my $easting = $ARGV[2] or die "No UTM easting specified at command line.\n";
	my $northing = $ARGV[3] or die "No UTM northing specified at command line.\n";
	($location_latitude, $location_longitude) = utm_to_latlon(23, $zone, $easting, $northing);
} elsif ($ARGV[0] eq "-latlon") {
	$location_latitude = $ARGV[1] or die "No latitude specified at command line.\n";
	$location_longitude = $ARGV[2] or die "No longitude specified at command line.\n";
} else {
	print "Usage:\tperl weatherc.pl [reference system] [coordinate(s)]\n";
	print "\tSupported reference systems: -mgrs, -utm, -latlon\n";
	print "\t-utm [Zone] [Easting] [Northing]\n";
	print "\t-mgrs [10-digit grid reference, preceeded by three-character zone and\n\t   two-letter prefix, without spaces.]\n";
	print "\t-latlon [Latitude] [Longitude]\n";
	exit;
}

my $parser = Text::CSV::Simple->new;
$parser->want_fields(0, 3, 4);
my @data = $parser->read_file($datafile);

# mmm...I guess technically if your point is the circumferance of the earth away that puts you right where you need to be ;)
my $smallestDistance = 2 * PI * R;
my $nearestZip = 0;

foreach (@data) {
	my $d = distance($location_latitude, $location_longitude, $_->[1], $_->[2]);
	if ($d < $smallestDistance)
	{
		$smallestDistance = $d;
		$nearestZip = $_->[0];
	}
}

#print "The point you gave me puts the closest ZIP code as " . $nearestZip . " at " . $smallestDistance . " miles away.\n";

# Now we're ready to get the weather for our point using Google Weather.
my $gw;
$gw = new Weather::Google($nearestZip);

my @info;
@info = $gw->current('temp_f','temp_c','humidity','wind_condition', 'condition');

print "Current conditions for " . $nearestZip . ":\n";
print "  Temperature: " . $info[0] . "\n";
print "  " . $info[2] . "\n";
print "  " . $info[3] . "\n";
print "  Condition: " . $info[4] . "\n";
